% 1 - Chargement de l'image
im = imread('image.png') ;
% Sous Octave, la ligne suivante n'est pas necessaire. Elle l est
% sous Matlab, qui ne charge pas directement l'image sous la forme
% d'un tableau de 0 et de 1.
im = sum(im,3) ; im = im/max(im(:)) ;

% 2 - Sous-echantillonnage
im_sub = im(1:2:end,1:2:end) ;

% 3 - Enregistrement
imwrite(im_sub,'im_sub.png') ;

% 4 - Flou local
% La commande circshift(im,[k,l]) decale (circulairement) le
% tableau im de k positions selon la premiere dimension et de l
% positions selon la deuxieme
im_blur_x = 0.25 * circshift(im,[1,0]) + 0.5 * im + ...
    0.25 * circshift(im,[-1,0]) ;
im_blur = 0.25 * circshift(im_blur_x,[0,1]) + 0.5 * im_blur_x + ...
          0.25 * circshift(im_blur_x,[0,-1]) ;

% 5 - Sous-echantillonnage et enregistrement
im_blur_sub = im_blur(1:2:end,1:2:end) ;
imwrite(im_blur_sub,'im_blur_sub.png') ;

% 6 - Troncature en frequence
% Definition de la transformee de Fourier de h
hf = zeros(178,336) ;
hf([1:45,135:178],[1:84,254:336]) = 1 ;
% Convolution avec im, en utilisant le fait que la transformee de
% Fourier transforme la convolution en produit
im_trunc = ifft2(hf.*fft2(im)) ;

% 7 - Projection sur les reels
fprintf(['La partie imaginaire a pour norme %.1e et la partie reelle ' ...
         '%.1e ; la partie imaginaire est bien negligeable\n'], ...
        norm(imag(im_trunc(:))),norm(real(im_trunc(:)))) ;
im_trunc = real(im_trunc) ;

% 8 - Sous-echantillonnage et enregistrement
im_trunc_sub = im_trunc(1:2:end,1:2:end) ;
imwrite(im_trunc_sub,'im_trunc_sub.png') ;

% 9 - Amelioration de h
% Les bavures sont dues au fait que le filtre h n'est pas bien
% localise en espace et oscille (c'est un produit de sinus
% cardinaux) et ce sont les oscillations qui provoquent les
% bavures. Pour remedier a cela, il faut definir un filtre h dont
% la transformee de Fourier est plus reguliere (par exemple
% continue ou C^1). Ainsi, h sera mieux localise en espace.

% Je definis : transformee de fourier de h en [k,l] = hx[k]*hy[l],
% avec hx et hy des fonctions "fenetres" assez regulieres

% hx[k] vaut 1 pour |k|<23, 0 pour |k|>44 et cos((|k|-a)*b) pour
% 22<|k|<45, avec a et b choisis de facon a ce que le filtre soit
% continu.
hx = [-88:89] ;
hx = (1+cos((abs(hx)-23)/22 * pi))/2 ...
     .* (23<=abs(hx)) .* (abs(hx)<=44) ...
     + 1 * (abs(hx)<23) ;
hx = circshift(hx,[0,-88]) ;

% De meme pour hy
hy = [-167:168] ;
hy = (1+cos((abs(hy)-41)/43 * pi))/2 ...
     .* (41<=abs(hy)) .* (abs(hy)<=83) ...
     + 1 * (abs(hy)<41) ;
hy = circshift(hy,[0,-167]) ;

% fourier(h)[k,l] = hx[k]*hy[l]
hf_v2 = hx.' * hy ;

% Convolution et partie reelle
im_trunc_v2 = ifft2(hf_v2.*fft2(im)) ;
im_trunc_v2 = real(im_trunc_v2) ;

% Sous-echantillonnage et enregistrement
im_trunc_sub_v2 = im_trunc_v2(1:2:end,1:2:end) ;
imwrite(im_trunc_sub_v2,'im_trunc_sub_v2.png') ;

% Le resultat est un peu plus flou que l'image obtenue dans la
% question 5 mais les oscillations ont disparu.